# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yggdrasil.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='yggdrasil.proto',
  package='yggdrasil',
  syntax='proto3',
  serialized_options=b'Z,github.com/redhatinsights/yggdrasil/protocol',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0fyggdrasil.proto\x12\tyggdrasil\"\x07\n\x05\x45mpty\"\xbe\x01\n\x13RegistrationRequest\x12\x0f\n\x07handler\x18\x01 \x01(\t\x12\x0b\n\x03pid\x18\x02 \x01(\x03\x12\x18\n\x10\x64\x65tached_content\x18\x03 \x01(\x08\x12>\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32,.yggdrasil.RegistrationRequest.FeaturesEntry\x1a/\n\rFeaturesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\";\n\x14RegistrationResponse\x12\x12\n\nregistered\x18\x01 \x01(\x08\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x01(\t\"\xb5\x01\n\x04\x44\x61ta\x12\x12\n\nmessage_id\x18\x01 \x01(\t\x12/\n\x08metadata\x18\x02 \x03(\x0b\x32\x1d.yggdrasil.Data.MetadataEntry\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\x12\x13\n\x0bresponse_to\x18\x04 \x01(\t\x12\x11\n\tdirective\x18\x05 \x01(\t\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\t\n\x07Receipt2\x8a\x01\n\nDispatcher\x12M\n\x08Register\x12\x1e.yggdrasil.RegistrationRequest\x1a\x1f.yggdrasil.RegistrationResponse\"\x00\x12-\n\x04Send\x12\x0f.yggdrasil.Data\x1a\x12.yggdrasil.Receipt\"\x00\x32\x37\n\x06Worker\x12-\n\x04Send\x12\x0f.yggdrasil.Data\x1a\x12.yggdrasil.Receipt\"\x00\x42.Z,github.com/redhatinsights/yggdrasil/protocolb\x06proto3'
)




_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='yggdrasil.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30,
  serialized_end=37,
)


_REGISTRATIONREQUEST_FEATURESENTRY = _descriptor.Descriptor(
  name='FeaturesEntry',
  full_name='yggdrasil.RegistrationRequest.FeaturesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='yggdrasil.RegistrationRequest.FeaturesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='yggdrasil.RegistrationRequest.FeaturesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=183,
  serialized_end=230,
)

_REGISTRATIONREQUEST = _descriptor.Descriptor(
  name='RegistrationRequest',
  full_name='yggdrasil.RegistrationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='handler', full_name='yggdrasil.RegistrationRequest.handler', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pid', full_name='yggdrasil.RegistrationRequest.pid', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='detached_content', full_name='yggdrasil.RegistrationRequest.detached_content', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='yggdrasil.RegistrationRequest.features', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_REGISTRATIONREQUEST_FEATURESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=230,
)


_REGISTRATIONRESPONSE = _descriptor.Descriptor(
  name='RegistrationResponse',
  full_name='yggdrasil.RegistrationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered', full_name='yggdrasil.RegistrationResponse.registered', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='address', full_name='yggdrasil.RegistrationResponse.address', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=232,
  serialized_end=291,
)


_DATA_METADATAENTRY = _descriptor.Descriptor(
  name='MetadataEntry',
  full_name='yggdrasil.Data.MetadataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='yggdrasil.Data.MetadataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='yggdrasil.Data.MetadataEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=428,
  serialized_end=475,
)

_DATA = _descriptor.Descriptor(
  name='Data',
  full_name='yggdrasil.Data',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message_id', full_name='yggdrasil.Data.message_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='yggdrasil.Data.metadata', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='content', full_name='yggdrasil.Data.content', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response_to', full_name='yggdrasil.Data.response_to', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='directive', full_name='yggdrasil.Data.directive', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DATA_METADATAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=294,
  serialized_end=475,
)


_RECEIPT = _descriptor.Descriptor(
  name='Receipt',
  full_name='yggdrasil.Receipt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=477,
  serialized_end=486,
)

_REGISTRATIONREQUEST_FEATURESENTRY.containing_type = _REGISTRATIONREQUEST
_REGISTRATIONREQUEST.fields_by_name['features'].message_type = _REGISTRATIONREQUEST_FEATURESENTRY
_DATA_METADATAENTRY.containing_type = _DATA
_DATA.fields_by_name['metadata'].message_type = _DATA_METADATAENTRY
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['RegistrationRequest'] = _REGISTRATIONREQUEST
DESCRIPTOR.message_types_by_name['RegistrationResponse'] = _REGISTRATIONRESPONSE
DESCRIPTOR.message_types_by_name['Data'] = _DATA
DESCRIPTOR.message_types_by_name['Receipt'] = _RECEIPT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'yggdrasil_pb2'
  # @@protoc_insertion_point(class_scope:yggdrasil.Empty)
  })
_sym_db.RegisterMessage(Empty)

RegistrationRequest = _reflection.GeneratedProtocolMessageType('RegistrationRequest', (_message.Message,), {

  'FeaturesEntry' : _reflection.GeneratedProtocolMessageType('FeaturesEntry', (_message.Message,), {
    'DESCRIPTOR' : _REGISTRATIONREQUEST_FEATURESENTRY,
    '__module__' : 'yggdrasil_pb2'
    # @@protoc_insertion_point(class_scope:yggdrasil.RegistrationRequest.FeaturesEntry)
    })
  ,
  'DESCRIPTOR' : _REGISTRATIONREQUEST,
  '__module__' : 'yggdrasil_pb2'
  # @@protoc_insertion_point(class_scope:yggdrasil.RegistrationRequest)
  })
_sym_db.RegisterMessage(RegistrationRequest)
_sym_db.RegisterMessage(RegistrationRequest.FeaturesEntry)

RegistrationResponse = _reflection.GeneratedProtocolMessageType('RegistrationResponse', (_message.Message,), {
  'DESCRIPTOR' : _REGISTRATIONRESPONSE,
  '__module__' : 'yggdrasil_pb2'
  # @@protoc_insertion_point(class_scope:yggdrasil.RegistrationResponse)
  })
_sym_db.RegisterMessage(RegistrationResponse)

Data = _reflection.GeneratedProtocolMessageType('Data', (_message.Message,), {

  'MetadataEntry' : _reflection.GeneratedProtocolMessageType('MetadataEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATA_METADATAENTRY,
    '__module__' : 'yggdrasil_pb2'
    # @@protoc_insertion_point(class_scope:yggdrasil.Data.MetadataEntry)
    })
  ,
  'DESCRIPTOR' : _DATA,
  '__module__' : 'yggdrasil_pb2'
  # @@protoc_insertion_point(class_scope:yggdrasil.Data)
  })
_sym_db.RegisterMessage(Data)
_sym_db.RegisterMessage(Data.MetadataEntry)

Receipt = _reflection.GeneratedProtocolMessageType('Receipt', (_message.Message,), {
  'DESCRIPTOR' : _RECEIPT,
  '__module__' : 'yggdrasil_pb2'
  # @@protoc_insertion_point(class_scope:yggdrasil.Receipt)
  })
_sym_db.RegisterMessage(Receipt)


DESCRIPTOR._options = None
_REGISTRATIONREQUEST_FEATURESENTRY._options = None
_DATA_METADATAENTRY._options = None

_DISPATCHER = _descriptor.ServiceDescriptor(
  name='Dispatcher',
  full_name='yggdrasil.Dispatcher',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=489,
  serialized_end=627,
  methods=[
  _descriptor.MethodDescriptor(
    name='Register',
    full_name='yggdrasil.Dispatcher.Register',
    index=0,
    containing_service=None,
    input_type=_REGISTRATIONREQUEST,
    output_type=_REGISTRATIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Send',
    full_name='yggdrasil.Dispatcher.Send',
    index=1,
    containing_service=None,
    input_type=_DATA,
    output_type=_RECEIPT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DISPATCHER)

DESCRIPTOR.services_by_name['Dispatcher'] = _DISPATCHER


_WORKER = _descriptor.ServiceDescriptor(
  name='Worker',
  full_name='yggdrasil.Worker',
  file=DESCRIPTOR,
  index=1,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=629,
  serialized_end=684,
  methods=[
  _descriptor.MethodDescriptor(
    name='Send',
    full_name='yggdrasil.Worker.Send',
    index=0,
    containing_service=None,
    input_type=_DATA,
    output_type=_RECEIPT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_WORKER)

DESCRIPTOR.services_by_name['Worker'] = _WORKER

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import yggdrasil_pb2 as yggdrasil__pb2


class DispatcherStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Register = channel.unary_unary(
                '/yggdrasil.Dispatcher/Register',
                request_serializer=yggdrasil__pb2.RegistrationRequest.SerializeToString,
                response_deserializer=yggdrasil__pb2.RegistrationResponse.FromString,
                )
        self.Send = channel.unary_unary(
                '/yggdrasil.Dispatcher/Send',
                request_serializer=yggdrasil__pb2.Data.SerializeToString,
                response_deserializer=yggdrasil__pb2.Receipt.FromString,
                )


class DispatcherServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Register(self, request, context):
        """Register is called by a worker to indicate it is ready and capable of
        handling the specified type of work.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Send(self, request, context):
        """Send is called by a worker to send data to the dispatcher.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DispatcherServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Register': grpc.unary_unary_rpc_method_handler(
                    servicer.Register,
                    request_deserializer=yggdrasil__pb2.RegistrationRequest.FromString,
                    response_serializer=yggdrasil__pb2.RegistrationResponse.SerializeToString,
            ),
            'Send': grpc.unary_unary_rpc_method_handler(
                    servicer.Send,
                    request_deserializer=yggdrasil__pb2.Data.FromString,
                    response_serializer=yggdrasil__pb2.Receipt.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'yggdrasil.Dispatcher', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Dispatcher(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Register(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yggdrasil.Dispatcher/Register',
            yggdrasil__pb2.RegistrationRequest.SerializeToString,
            yggdrasil__pb2.RegistrationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Send(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yggdrasil.Dispatcher/Send',
            yggdrasil__pb2.Data.SerializeToString,
            yggdrasil__pb2.Receipt.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class WorkerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Send = channel.unary_unary(
                '/yggdrasil.Worker/Send',
                request_serializer=yggdrasil__pb2.Data.SerializeToString,
                response_deserializer=yggdrasil__pb2.Receipt.FromString,
                )


class WorkerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Send(self, request, context):
        """Send is called by the dispatcher to send data to a worker.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_WorkerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Send': grpc.unary_unary_rpc_method_handler(
                    servicer.Send,
                    request_deserializer=yggdrasil__pb2.Data.FromString,
                    response_serializer=yggdrasil__pb2.Receipt.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'yggdrasil.Worker', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Worker(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Send(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/yggdrasil.Worker/Send',
            yggdrasil__pb2.Data.SerializeToString,
            yggdrasil__pb2.Receipt.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import sys
import os
from .constants import WORKER_LIB_DIR, CONFIG_FILE
sys.path.insert(0, WORKER_LIB_DIR)
import toml
import yaml
import grpc
import ansible_runner
import time
import json
import uuid
import atexit
from subprocess import Popen, PIPE
from requests import Request
from concurrent import futures
from .protocol import yggdrasil_pb2_grpc, yggdrasil_pb2
from .dispatcher_events import executor_on_start, executor_on_failed

# unbuffered stdout for logging to rhc
sys.stdout = os.fdopen(sys.stdout.fileno(), 'wb', buffering=0)
atexit.register(sys.stdout.close)

def _log(message):
    '''
    Send message as bytes over unbuffered stdout for
    RHC to log
    '''
    sys.stdout.write((message + '\n').encode())

# for some reason, without a short delay, RHC connects too quickly
#   and cloud connector service can't see it
time.sleep(5)

YGG_SOCKET_ADDR = os.environ.get('YGG_SOCKET_ADDR')
if not YGG_SOCKET_ADDR:
    _log("Missing YGG_SOCKET_ADDR environment variable")
    sys.exit(1)
# massage the value for python grpc
YGG_SOCKET_ADDR = YGG_SOCKET_ADDR.replace("unix:@", "unix-abstract:")
BASIC_PATH = "/sbin:/bin:/usr/sbin:/usr/bin"

def _newlineDelimited(events):
    '''
    Dump a list into a newline-delimited JSON format
    '''
    output = ''
    for e in events:
        output += json.dumps(e) + '\n'
    return output

def _generateRequest(events, return_url):
    '''
    Generate the HTTP request
    '''
    # TODO?: generate by hand so request isn't a dependency
    return Request('POST', return_url, files={
        "file": ("runner-events", _newlineDelimited(events), "application/vnd.redhat.playbook.v1+jsonl"),
        "metadata": "{}"
    }).prepare()

def _composeDispatcherMessage(events, return_url, response_to):
    '''
    Create the message with event data to send back to Dispatcher
    '''
    req = _generateRequest(events, return_url)
    return yggdrasil_pb2.Data(
        message_id=str(uuid.uuid4()).encode('utf-8'),
        content=req.body,
        directive=return_url,
        metadata=req.headers,
        response_to=response_to)

def _parseFailure(event):
    '''
    Generate the error code and details from the failure event
    '''
    errorCode = "UNDEFINED_ERROR"
    errorDetails = event.get('stdout')
    if "The command was not found or was not executable: ansible-playbook" in errorDetails:
        errorCode = "ANSIBLE_PLAYBOOK_NOT_INSTALLED"
    # TODO: enumerate more failure types
    return errorCode, errorDetails

def _loadConfig():
    # load config file
    _config = {}
    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE) as _f:
            _config = toml.load(_f)
    else:
        _log("WARNING: Config file does not exist: %s. Using defaults." % CONFIG_FILE)

    parsedConfig = {
        "directive": _config.get("directive", "rhc-worker-playbook"),
        "verify_playbook": _config.get("verify_playbook", True),
        "verify_playbook_version_check": _config.get("verify_playbook_version_check", True),
        "insights_core_gpg_check": _config.get("insights_core_gpg_check", True)
    }
    return parsedConfig


class Events(list):
    '''
    Extension of list to receive ansible-runner events
    '''
    def __init__(self):
        pass

    def addEvent(self, event):
        try:
            event.get("event_data", {}).get("res", {}).pop("ansible_facts", None)
        except AttributeError:
            # one of the fields was null/empty
            pass
        self.append(event)


class WorkerService(yggdrasil_pb2_grpc.WorkerServicer):

    def __init__(self, *args, **kwargs):
        dispatcher = kwargs.get('dispatcher', None)
        if not dispatcher:
            _log('No dispatcher parameter was provided to the WorkerService')
            raise Exception
        self.dispatcher = dispatcher

    def Send(self, request, context):
        '''
        Act on messages sent to the WorkerService
        '''
        # we have received it
        yggdrasil_pb2.Receipt()

        # load configuration
        config = _loadConfig()

        events = Events()
        # parse playbook from data field
        try:
            # required fields
            playbook_str = request.content
            response_to = request.message_id
            crc_dispatcher_correlation_id = request.metadata.get('crc_dispatcher_correlation_id')
            response_interval = request.metadata.get('response_interval')
            return_url = request.metadata.get('return_url')
        except LookupError as e:
            _log("ERROR: Missing attribute in message: %s" % e)
            raise

        if not crc_dispatcher_correlation_id:
            _log("WARNING: No crc_dispatcher_correlation_id")

        if not return_url:
            _log("WARNING: No return_url")

        if not response_interval:
            _log("WARNING: No response_interval. Defaulting to 300")
            response_interval = 300
        else:
            try:
                response_interval = float(response_interval)
            except (TypeError, ValueError) as e:
                _log("ERROR: Invalid response_interval")
                _log(str(e))
                raise

        if config["verify_playbook"]:
            _log("Verifying playbook...")
            # --payload here will be a no-op because no upload is performed when using the verifier
            #   but, it will allow us to update the egg!
            args = ["insights-client", "-m", "insights.client.apps.ansible.playbook_verifier",
                    "--quiet", "--payload", "noop", "--content-type", "noop"]
            env = {"PATH": BASIC_PATH}
            if config["insights_core_gpg_check"] == False:
                args.append("--no-gpg")
                env["BYPASS_GPG"] = "True"
            verifyProc = Popen(
                args,
                stdin=PIPE, stdout=PIPE, stderr=PIPE,
                env=env)
            stdout, stderr = verifyProc.communicate(input=playbook_str)
            if verifyProc.returncode != 0:
                _log("ERROR: Unable to verify playbook:\n%s\n%s" %
                (stdout.decode("utf-8"), stderr.decode("utf-8")))
                raise Exception
            verified = stdout.decode("utf-8")
            _log("Playbook verified.")
        else:
            _log("WARNING: Playbook verification disabled.")
            verified = playbook_str.decode("utf-8")
        try:
            playbook = yaml.safe_load(verified)
        except yaml.composer.ComposerError as e:
            _log("ERROR: Could not parse playbook")
            _log(str(e))
            raise

        for item in playbook:
            # remove signature field, ansible-runner dislikes bytes
            try:
                item.get('vars', {}).pop('insights_signature', None)
            except AttributeError:
                # vars was null/empty
                pass

        _log("Starting playbook run...")
        # required event for cloud connector
        on_start = executor_on_start(correlation_id=crc_dispatcher_correlation_id)
        events.addEvent(on_start)

        # run playbook
        runnerThread, runner = ansible_runner.interface.run_async(
            playbook=playbook,
            envvars={"PYTHONPATH": WORKER_LIB_DIR, "PATH": BASIC_PATH},
            event_handler=events.addEvent,
            quiet=True)

        # initialize elapsed counter
        elapsedTime = 0
        startTime = time.time()
        while runnerThread.is_alive():
            elapsedTime = time.time() - startTime
            if elapsedTime >= response_interval:
                # hit the interval, post events
                _log("Hit the response interval. Posting current status...")
                returnedEvents = _composeDispatcherMessage(events, return_url, response_to)
                response = self.dispatcher.Send(returnedEvents)
                # reset interval timer
                elapsedTime = 0
                startTime = time.time()

        if runner.status == 'failed':
            # last event sould be the failure, find the reason
            errorCode, errorDetails = _parseFailure(events[-1])
            if errorCode == "ANSIBLE_PLAYBOOK_NOT_INSTALLED":
                _log("WARNING: The rhc-worker-playbook package requires the ansible package to be installed.")
            # required event for cloud connector
            on_failed = executor_on_failed(correlation_id=crc_dispatcher_correlation_id, error_code=errorCode, error_details=errorDetails)
            events.addEvent(on_failed)

        # send the final message after playbook completed
        _log("Playbook run complete.")
        _log(str(events))
        returnedEvents = _composeDispatcherMessage(events, return_url, response_to)
        _log("Posting events...")
        response = self.dispatcher.Send(returnedEvents)
        _log("Post complete.")
        return

def serve():
    # load config to get directive
    config = _loadConfig()

    # open the channel to ygg Dispatcher
    channel = grpc.insecure_channel(YGG_SOCKET_ADDR)
    dispatcher = yggdrasil_pb2_grpc.DispatcherStub(channel)
    _log("Registering with directive %s..." % config["directive"])
    registrationResponse = dispatcher.Register(
        yggdrasil_pb2.RegistrationRequest(
            handler=config["directive"],
            detached_content=True,
            pid=os.getpid()))
    registered = registrationResponse.registered
    if not registered:
        _log("ERROR: Could not register rhc-worker-playbook.")
        sys.exit(1)
    _log("Registered rhc-worker-playbook.")
    address = registrationResponse.address.replace("@", "unix-abstract:")

    # create server
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    try:
        yggdrasil_pb2_grpc.add_WorkerServicer_to_server(WorkerService(dispatcher=dispatcher), server)
    except ValueError as e:
        _log(str(e))
        raise
    server.add_insecure_port(address)

    # off to the races
    server.start()
    server.wait_for_termination()


if __name__ == '__main__':
    serve()



import setuptools

setuptools.setup(
    name="rhc-worker-playbook",
    version="0.1.5",
    author="Jeremy Crafts",
    author_email="jcrafts@redhat.com",
    description="Python worker for RHC",
    long_description="Listens on gRPC messages and launches Ansible with received playbooks",
    url="https://github.com/RedHatInsights/rhc-ansible-worker",
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    scripts=[
        "scripts/rhc-worker-playbook.worker"
    ],
    zip_safe = False,
)



'''
Event-generating functions for the Playbook Dispatcher
'''
import uuid

# required event for cloud connector
def executor_on_start(correlation_id=None, stdout=None):
    return {
        "event": "executor_on_start",
        "uuid": str(uuid.uuid4()),
        "counter": -1,
        "stdout": stdout,
        "start_line": 0,
        "end_line": 0,
        "event_data": {
            "crc_dispatcher_correlation_id": correlation_id
        }
    }

# required event for cloud connector
def executor_on_failed(correlation_id=None, stdout=None, error_code=None, error_details=None):
    return {
        "event": "executor_on_failed",
        "uuid": str(uuid.uuid4()),
        "counter": -1,
        "stdout": stdout,
        "start_line": 0,
        "end_line": 0,
        "event_data": {
            "crc_dispatcher_correlation_id": correlation_id,
            "crc_dispatcher_error_code": error_code,
            "crc_dispatcher_error_details": error_details
        }
    }